@@ -1,6 +1,6 @@
 /* avenprcore.h
  * Header file for printer independent parts of Survex printer drivers
- * Copyright (C) 1994-2002,2004,2005,2012,2013,2014 Olly Betts
+ * Copyright (C) 1994-2002,2004,2005,2012,2013,2014,2015 Olly Betts
  * Copyright (C) 2004 Philip Underwood
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,32 +21,16 @@
 #ifndef survex_included_avenprcore_h
 #define survex_included_avenprcore_h
 
-#include "img_hosted.h"
 #include <wx.h>
 
-extern bool fBlankPage;
-
-struct border {
-   long x_min, y_min, x_max, y_max;
-};
-
-#define PR_FONT_DEFAULT	0
-#define PR_FONT_LABELS	1
-
-#define PR_COLOUR_TEXT		0
-#define PR_COLOUR_LABELS	1
-#define PR_COLOUR_FRAME		2
-#define PR_COLOUR_LEG		3
-#define PR_COLOUR_CROSS		4
-#define PR_COLOUR_SURFACE_LEG	5
-
-#define PR_FLAG_NOFILEOUTPUT	1
-#define PR_FLAG_NOINI		2
-#define PR_FLAG_CALIBRATE	4
-
 /* 1:<DEFAULT_SCALE> is the default scale */
 #define DEFAULT_SCALE 500
 
+/* Height of our footer in mm. */
+#define FOOTER_HEIGHT_MM 10
+
+class wxPageSetupDialogData;
+
 /* Store everything describing the page layout */
 class layout {
 public:
@@ -61,7 +45,7 @@ public:
     wxString datestamp;
     time_t datestamp_numeric;
     double Scale;
-    int rot, tilt;
+    double rot, tilt;
     enum {PLAN, ELEV, TILT, EXTELEV} view;
 
     /* internal data, but may be accessed */
@@ -70,9 +54,8 @@ public:
     double PaperWidth, PaperDepth;
     int pagesX, pagesY, pages;
     double xOrg, yOrg;
-    wxString footer;
 
-    layout(wxPageSetupDialogData* data);
+    explicit layout(wxPageSetupDialogData* data);
 #if 0
     void make_calibration();
 #endif
@@ -80,21 +63,4 @@ public:
     void pages_required();
 };
 
-/* things for a back end */
-void drawticks(border clip, int tick_size, int x, int y);
-
-int as_int(const char *v, char *p, int min_val, int max_val);
-unsigned long as_colour(const char *v, char *p);
-int as_bool(const char *v, char *p);
-double as_double(const char *v, char *p, double min_val, double max_val);
-int as_escstring(const char *v, char *s);
-char *as_string(const char *v, char *p);
-
-#if 0
-class MainFrm;
-void print_all(MainFrm *m_parent, layout *l, device *pri);
-void print_page(MainFrm *m_parent, layout *l, int page, int pass, int cPasses);
-int next_page(int *pstate, char **q, int pageLim);
-#endif
-
 #endif
