@@ -1,7 +1,7 @@
 /* cavernlog.cc
  * Run cavern inside an Aven window
  *
- * Copyright (C) 2005,2006,2010,2011,2012,2014,2015 Olly Betts
+ * Copyright (C) 2005,2006,2010,2011,2012,2014,2015,2016 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,37 +40,156 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include <wx/process.h>
+
 enum { LOG_REPROCESS = 1234, LOG_SAVE = 1235 };
 
+static const wxString badutf8_html(
+    wxT("<span style=\"color:white;background-color:red;\">&#xfffd;</span>"));
+static const wxString badutf8(wxUniChar(0xfffd));
+
+// New event type for passing a chunk of cavern output from the worker thread
+// to the main thread (or from the idle event handler if we're not using
+// threads).
+class CavernOutputEvent;
+
+wxDEFINE_EVENT(wxEVT_CAVERN_OUTPUT, CavernOutputEvent);
+
+class CavernOutputEvent : public wxEvent {
+  public:
+    char buf[1000];
+    int len;
+    CavernOutputEvent() : wxEvent(0, wxEVT_CAVERN_OUTPUT), len(0) { }
+
+    wxEvent * Clone() const {
+	CavernOutputEvent * e = new CavernOutputEvent();
+	e->len = len;
+	if (len > 0) memcpy(e->buf, buf, len);
+	return e;
+    }
+};
+
+#ifdef CAVERNLOG_USE_THREADS
+class CavernThread : public wxThread {
+  protected:
+    virtual ExitCode Entry();
+
+    CavernLogWindow *handler;
+
+    wxInputStream * in;
+
+  public:
+    CavernThread(CavernLogWindow *handler_, wxInputStream * in_)
+	: wxThread(wxTHREAD_DETACHED), handler(handler_), in(in_) { }
+
+    ~CavernThread() {
+	wxCriticalSectionLocker enter(handler->thread_lock);
+	handler->thread = NULL;
+    }
+};
+
+wxThread::ExitCode
+CavernThread::Entry()
+{
+    while (true) {
+	CavernOutputEvent * e = new CavernOutputEvent();
+	in->Read(e->buf, sizeof(e->buf));
+	size_t n = in->LastRead();
+	if (n == 0 || TestDestroy()) {
+	    delete e;
+	    return (wxThread::ExitCode)0;
+	}
+	if (n == 1 && e->buf[0] == '\n') {
+	    // Don't send an event with just a blank line in.
+	    in->Read(e->buf + 1, sizeof(e->buf) - 1);
+	    n += in->LastRead();
+	    if (TestDestroy()) {
+		delete e;
+		return (wxThread::ExitCode)0;
+	    }
+	}
+	e->len = n;
+	handler->QueueEvent(e);
+    }
+}
+
+#else
+
+void
+CavernLogWindow::OnIdle(wxIdleEvent& event)
+{
+    if (cavern_out == NULL) return;
+
+    wxInputStream * in = cavern_out->GetInputStream();
+
+    if (!in->CanRead()) {
+	// Avoid a tight busy-loop on idle events.
+	wxMilliSleep(10);
+    }
+    if (in->CanRead()) {
+	CavernOutputEvent * e = new CavernOutputEvent();
+	in->Read(e->buf, sizeof(e->buf));
+	size_t n = in->LastRead();
+	if (n == 0) {
+	    delete e;
+	    return;
+	}
+	e->len = n;
+	QueueEvent(e);
+    }
+
+    event.RequestMore();
+}
+#endif
+
 BEGIN_EVENT_TABLE(CavernLogWindow, wxHtmlWindow)
     EVT_BUTTON(LOG_REPROCESS, CavernLogWindow::OnReprocess)
     EVT_BUTTON(LOG_SAVE, CavernLogWindow::OnSave)
     EVT_BUTTON(wxID_OK, CavernLogWindow::OnOK)
+    EVT_COMMAND(wxID_ANY, wxEVT_CAVERN_OUTPUT, CavernLogWindow::OnCavernOutput)
+#ifdef CAVERNLOG_USE_THREADS
+    EVT_CLOSE(CavernLogWindow::OnClose)
+#else
+    EVT_IDLE(CavernLogWindow::OnIdle)
+#endif
+    EVT_END_PROCESS(wxID_ANY, CavernLogWindow::OnEndProcess)
 END_EVENT_TABLE()
 
 static wxString escape_for_shell(wxString s, bool protect_dash = false)
 {
-    size_t p = 0;
 #ifdef __WXMSW__
-    bool needs_quotes = false;
-    while (p < s.size()) {
-	wxChar ch = s[p];
-	if (ch < 127) {
-	    if (ch == wxT('"')) {
-		s.insert(p, 1, wxT('\\'));
-		++p;
-		needs_quotes = true;
-	    } else if (strchr(" <>&|^", ch)) {
-		needs_quotes = true;
+    // Correct quoting rules are insane:
+    //
+    // http://blogs.msdn.com/b/twistylittlepassagesallalike/archive/2011/04/23/everyone-quotes-arguments-the-wrong-way.aspx
+    //
+    // Thankfully wxExecute passes the command string to CreateProcess(), so
+    // at least we don't need to quote for cmd.exe too.
+    if (s.empty() || s.find_first_of(wxT(" \"\t\n\v")) != s.npos) {
+	// Need to quote.
+	s.insert(0, wxT('"'));
+	for (size_t p = 1; p < s.size(); ++p) {
+	    size_t backslashes = 0;
+	    while (s[p] == wxT('\\')) {
+		++backslashes;
+		if (++p == s.size()) {
+		    // Escape all the backslashes, since they're before
+		    // the closing quote we add below.
+		    s.append(backslashes, wxT('\\'));
+		    goto done;
+		}
+	    }
+
+	    if (s[p] == wxT('"')) {
+		// Escape any preceding backslashes and this quote.
+		s.insert(p, backslashes + 1, wxT('\\'));
+		p += backslashes + 1;
 	    }
 	}
-	++p;
-    }
-    if (needs_quotes) {
-	s.insert(0u, 1, wxT('"'));
-	s += wxT('"');
+done:
+	s.append(wxT('"'));
     }
 #else
+    size_t p = 0;
     if (protect_dash && !s.empty() && s[0u] == '-') {
 	// If the filename starts with a '-', protect it from being
 	// treated as an option by prepending "./".
@@ -90,33 +209,104 @@ static wxString escape_for_shell(wxString s, bool protect_dash = false)
 }
 
 CavernLogWindow::CavernLogWindow(MainFrm * mainfrm_, const wxString & survey_, wxWindow * parent)
-    : wxHtmlWindow(parent), mainfrm(mainfrm_), init_done(false), survey(survey_)
+    : wxHtmlWindow(parent), mainfrm(mainfrm_), cavern_out(NULL),
+      link_count(0), end(buf), init_done(false), survey(survey_)
+#ifdef CAVERNLOG_USE_THREADS
+      , thread(NULL)
+#endif
 {
     int fsize = parent->GetFont().GetPointSize();
     int sizes[7] = { fsize, fsize, fsize, fsize, fsize, fsize, fsize };
     SetFonts(wxString(), wxString(), sizes);
 }
 
+CavernLogWindow::~CavernLogWindow()
+{
+#ifdef CAVERNLOG_USE_THREADS
+    if (thread) stop_thread();
+#endif
+    if (cavern_out) {
+	wxEndBusyCursor();
+	cavern_out->Detach();
+    }
+}
+
+#ifdef CAVERNLOG_USE_THREADS
+void
+CavernLogWindow::stop_thread()
+{
+    // Killing the subprocess by its pid is theoretically racy, but in practice
+    // it's not going to cause issues, and it's all the wxProcess API seems to
+    // allow us to do.  If we don't kill the subprocess, we need to wait for it
+    // to write out some output - there seems to be no way to do the equivalent
+    // of select() with a timeout on a a wxInputStream.
+    //
+    // The only alternative to this seems to be to do:
+    //
+    //     while (!s.CanRead()) {
+    //         if (TestDestroy()) return (wxThread::ExitCode)0;
+    //         wxMilliSleep(N);
+    //     }
+    //
+    // But that makes the log window update sluggishly, and we're using a
+    // worker thread precisely to try to avoid having to do dumb stuff like
+    // this.
+    wxProcess::Kill(cavern_out->GetPid());
+
+    {
+	wxCriticalSectionLocker enter(thread_lock);
+	if (thread) {
+	    wxThreadError res;
+#if wxCHECK_VERSION(2,9,2)
+	    res = thread->Delete(NULL, wxTHREAD_WAIT_BLOCK);
+#else
+	    res = thread->Delete();
+#endif
+	    if (res != wxTHREAD_NO_ERROR) {
+		// FIXME
+	    }
+	}
+    }
+
+    // Wait for thread to complete.
+    while (true) {
+	{
+	    wxCriticalSectionLocker enter(thread_lock);
+	    if (!thread) break;
+	}
+	wxMilliSleep(1);
+    }
+}
+
+void
+CavernLogWindow::OnClose(wxCloseEvent &)
+{
+    if (thread) stop_thread();
+    Destroy();
+}
+#endif
+
 void
 CavernLogWindow::OnLinkClicked(const wxHtmlLinkInfo &link)
 {
     wxString href = link.GetHref();
     wxString title = link.GetTarget();
-    size_t colon = href.rfind(wxT(':'));
+    size_t colon2 = href.rfind(wxT(':'));
+    if (colon2 == wxString::npos)
+	return;
+    size_t colon = href.rfind(wxT(':'), colon2 - 1);
     if (colon == wxString::npos)
 	return;
-    size_t colon2 = href.rfind(wxT(':'), colon - 1);
-    if (colon2 != wxString::npos) swap(colon, colon2);
 #ifdef __WXMSW__
     wxString cmd = wxT("notepad $f");
 #elif defined __WXMAC__
     wxString cmd = wxT("open -t $f");
 #else
-    wxString cmd = wxT("x-terminal-emulator -title $t -e vim +'call cursor($l,$c)' $f");
+    wxString cmd = wxT("x-terminal-emulator -e vim +'call cursor($l,$c)' $f");
     // wxString cmd = wxT("gedit -b $f +$l:$c $f");
-    // wxString cmd = wxT("x-terminal-emulator -title $t -e emacs +$l $f");
-    // wxString cmd = wxT("x-terminal-emulator -title $t -e nano +$l $f");
-    // wxString cmd = wxT("x-terminal-emulator -title $t -e jed -g $l $f");
+    // wxString cmd = wxT("x-terminal-emulator -e emacs +$l $f");
+    // wxString cmd = wxT("x-terminal-emulator -e nano +$l $f");
+    // wxString cmd = wxT("x-terminal-emulator -e jed -g $l $f");
 #endif
     wxChar * p = wxGetenv(wxT("SURVEXEDITOR"));
     if (p) {
@@ -152,7 +342,7 @@ CavernLogWindow::OnLinkClicked(const wxHtmlLinkInfo &link)
 	    }
 	    case wxT('c'): {
 		wxString l;
-		if (colon2 == wxString::npos)
+		if (colon2 >= href.size())
 		    l = wxT("0");
 		else
 		    l = escape_for_shell(href.substr(colon2 + 1));
@@ -164,8 +354,10 @@ CavernLogWindow::OnLinkClicked(const wxHtmlLinkInfo &link)
 		++i;
 	}
     }
-    if (wxSystem(cmd) >= 0)
+
+    if (wxExecute(cmd, wxEXEC_ASYNC|wxEXEC_MAKE_GROUP_LEADER) >= 0)
 	return;
+
     wxString m;
     // TRANSLATORS: %s is replaced by the command we attempted to run.
     m.Printf(wmsg(/*Couldn’t run external command: “%s”*/17), cmd.c_str());
@@ -175,12 +367,25 @@ CavernLogWindow::OnLinkClicked(const wxHtmlLinkInfo &link)
     wxGetApp().ReportError(m);
 }
 
-int
+void
 CavernLogWindow::process(const wxString &file)
 {
+    SetPage(wxString());
+#ifdef CAVERNLOG_USE_THREADS
+    if (thread) stop_thread();
+#endif
+    if (cavern_out) {
+	cavern_out->Detach();
+	cavern_out = NULL;
+    } else {
+	wxBeginBusyCursor();
+    }
+
     SetFocus();
     filename = file;
 
+    link_count = 0;
+    cur.resize(0);
     log_txt.resize(0);
 
 #ifdef __WXMSW__
@@ -203,11 +408,13 @@ CavernLogWindow::process(const wxString &file)
 	    len += len;
 	}
 	/* Strange Win32 nastiness - strip prefix "\\?\" if present */
-	if (wcsncmp(buf, L"\\\\?\\", 4) == 0) buf += 4;
-	wchar_t * slash = wcsrchr(buf, L'\\');
+	wchar_t *start = buf;
+	if (wcsncmp(start, L"\\\\?\\", 4) == 0) start += 4;
+	wchar_t * slash = wcsrchr(start, L'\\');
 	if (slash) {
-	    cmd.assign(buf, slash - buf + 1);
+	    cmd.assign(start, slash - start + 1);
 	}
+	osfree(buf);
     }
     cmd += L"cavern";
     cmd = escape_for_shell(cmd, false);
@@ -221,11 +428,7 @@ CavernLogWindow::process(const wxString &file)
     cmd += wxT(' ');
     cmd += escaped_file;
 
-#ifdef __WXMSW__
-    FILE * cavern_out = _wpopen(cmd.c_str(), L"r");
-#else
-    FILE * cavern_out = popen(cmd.mb_str(), "r");
-#endif
+    cavern_out = wxProcess::Open(cmd);
     if (!cavern_out) {
 	wxString m;
 	m.Printf(wmsg(/*Couldn’t run external command: “%s”*/17), cmd.c_str());
@@ -233,64 +436,33 @@ CavernLogWindow::process(const wxString &file)
 	m += wxString(strerror(errno), wxConvUTF8);
 	m += wxT(')');
 	wxGetApp().ReportError(m);
-	return -2;
+	return;
     }
 
-    const wxString & error_marker = wmsg(/*error*/93) + ":";
-    const wxString & warning_marker = wmsg(/*warning*/4) + ":";
+    // We want to receive the wxProcessEvent when cavern exits.
+    cavern_out->SetNextHandler(this);
 
-    int cavern_fd;
-#ifdef __WXMSW__
-    cavern_fd = _fileno(cavern_out);
-#else
-    cavern_fd = fileno(cavern_out);
+#ifdef CAVERNLOG_USE_THREADS
+    thread = new CavernThread(this, cavern_out->GetInputStream());
+    if (thread->Run() != wxTHREAD_NO_ERROR) {
+	wxGetApp().ReportError(wxT("Thread failed to start"));
+	delete thread;
+	thread = NULL;
+    }
 #endif
-    assert(cavern_fd < FD_SETSIZE); // FIXME we shouldn't just assert, but what else to do?
-    wxString cur;
-    int link_count = 0;
-    unsigned char buf[1024];
-    unsigned char * end = buf;
-    while (true) {
-	fd_set rfds, efds;
-	FD_ZERO(&rfds);
-	FD_SET(cavern_fd, &rfds);
-	FD_ZERO(&efds);
-	FD_SET(cavern_fd, &efds);
-	// Timeout instantly the first time, and call Update() so the window
-	// contents get updated.  If we're still waiting for output after that,
-	// wait for output for 0.1 seconds, then call wxYield() so that the UI
-	// doesn't block.
-	struct timeval timeout;
-	timeout.tv_sec = 0;
-	timeout.tv_usec = 0;
-	bool first = true;
-	while (select(cavern_fd + 1, &rfds, NULL, &efds, &timeout) == 0) {
-	    if (first) {
-		first = false;
-		Update();
-	    } else {
-		wxYield();
-	    }
-	    FD_SET(cavern_fd, &rfds);
-	    FD_SET(cavern_fd, &efds);
-	    // Set timeout to 0.1 seconds.
-	    timeout.tv_sec = 0;
-	    timeout.tv_usec = 100000;
-	}
-	if (!FD_ISSET(cavern_fd, &rfds)) {
-	    // Error, which pclose() should report.
-	    break;
-	}
+}
 
-	ssize_t r = read(cavern_fd, end, sizeof(buf) - (end - buf));
-	if (r <= 0) {
-	    if (r == 0 && buf != end) {
-		// FIXME: Truncated UTF-8 sequence.
-	    }
-	    break;
-	}
-	log_txt.append((const char *)end, r);
-	end += r;
+void
+CavernLogWindow::OnCavernOutput(wxCommandEvent & e_)
+{
+    CavernOutputEvent & e = (CavernOutputEvent&)e_;
+
+    if (e.len > 0) {
+	ssize_t n = e.len;
+	if (size_t(n) > sizeof(buf) - (end - buf)) abort();
+	memcpy(end, e.buf, n);
+	log_txt.append((const char *)end, n);
+	end += n;
 
 	const unsigned char * p = buf;
 
@@ -299,8 +471,8 @@ CavernLogWindow::process(const wxString &file)
 	    if (ch >= 0x80) {
 		// Decode multi-byte UTF-8 sequence.
 		if (ch < 0xc0) {
-		    // FIXME: Invalid UTF-8 sequence.
-		    goto abort;
+		    // Invalid UTF-8 sequence.
+		    goto bad_utf8;
 		} else if (ch < 0xe0) {
 		    /* 2 byte sequence */
 		    if (p == end) {
@@ -309,8 +481,8 @@ CavernLogWindow::process(const wxString &file)
 		    }
 		    int ch1 = *p++;
 		    if ((ch1 & 0xc0) != 0x80) {
-			// FIXME: Invalid UTF-8 sequence.
-			goto abort;
+			// Invalid UTF-8 sequence.
+			goto bad_utf8;
 		    }
 		    ch = ((ch & 0x1f) << 6) | (ch1 & 0x3f);
 		} else if (ch < 0xf0) {
@@ -322,21 +494,32 @@ CavernLogWindow::process(const wxString &file)
 		    int ch1 = *p++;
 		    ch = ((ch & 0x1f) << 12) | ((ch1 & 0x3f) << 6);
 		    if ((ch1 & 0xc0) != 0x80) {
-			// FIXME: Invalid UTF-8 sequence.
-			goto abort;
+			// Invalid UTF-8 sequence.
+			goto bad_utf8;
 		    }
 		    int ch2 = *p++;
 		    if ((ch2 & 0xc0) != 0x80) {
-			// FIXME: Invalid UTF-8 sequence.
-			goto abort;
+			// Invalid UTF-8 sequence.
+			goto bad_utf8;
 		    }
 		    ch |= (ch2 & 0x3f);
 		} else {
-		    // FIXME: Overlong UTF-8 sequence.
-		    goto abort;
+		    // Overlong UTF-8 sequence.
+		    goto bad_utf8;
 		}
 	    }
 
+	    if (false) {
+bad_utf8:
+		// Resync to next byte which starts a UTF-8 sequence.
+		while (p != end) {
+		    if (*p < 0x80 || (*p >= 0xc0 && *p < 0xf0)) break;
+		    ++p;
+		}
+		cur += badutf8_html;
+		continue;
+	    }
+
 	    switch (ch) {
 		case '\r':
 		    // Ignore.
@@ -364,17 +547,28 @@ CavernLogWindow::process(const wxString &file)
 			       cur[i] >= wxT('0') && cur[i] <= wxT('9')) { }
 			    if (i > colon + 1 && cur[i] == wxT(':') ) {
 				colon = i;
+			    } else {
+				// If there's no colon, include a trailing ':'
+				// so that we can unambiguously split the href
+				// value up into filename, line and column.
+				++colon;
 			    }
 			    wxString tag = wxT("<a href=\"");
 			    tag.append(cur, 0, colon);
 			    while (cur[++i] == wxT(' ')) { }
 			    tag += wxT("\" target=\"");
-			    tag.append(cur, i, wxString::npos);
+			    wxString target(cur, i, wxString::npos);
+			    target.Replace(badutf8_html, badutf8);
+			    tag += target;
 			    tag += wxT("\">");
 			    cur.insert(0, tag);
 			    size_t offset = colon + tag.size();
 			    cur.insert(offset, wxT("</a>"));
 			    offset += 4 + 2;
+
+			    static const wxString & error_marker = wmsg(/*error*/93) + ":";
+			    static const wxString & warning_marker = wmsg(/*warning*/4) + ":";
+
 			    if (cur.substr(offset, error_marker.size()) == error_marker) {
 				// Show "error" marker in red.
 				cur.insert(offset, wxT("<span style=\"color:red\">"));
@@ -399,8 +593,8 @@ CavernLogWindow::process(const wxString &file)
 		    AppendToPage(cur);
 
 		    if (!link_count) {
-			// Auto-scroll the window until we've reported a warning or
-			// error.
+			// Auto-scroll the window until we've reported a
+			// warning or error.
 			int x, y;
 			GetVirtualSize(&x, &y);
 			int xs, ys;
@@ -441,28 +635,31 @@ CavernLogWindow::process(const wxString &file)
 	size_t left = end - p;
 	end = buf + left;
 	if (left) memmove(buf, p, left);
+	Update();
+	return;
+    }
+
+    if (e.len <= 0 && buf != end) {
+	// Truncated UTF-8 sequence.
+	cur += badutf8_html;
     }
-abort:
 
     /* TRANSLATORS: Label for button in aven’s cavern log window which
      * allows the user to save the log to a file. */
     AppendToPage(wxString::Format(wxT("<avenbutton id=%d name=\"%s\">"),
 				  (int)LOG_SAVE,
 				  wmsg(/*Save Log*/446).c_str()));
-    int retval = pclose(cavern_out);
-    if (retval) {
+    wxEndBusyCursor();
+    delete cavern_out;
+    cavern_out = NULL;
+    if (e.len < 0) {
+	/* Negative length indicates non-zero exit status from cavern. */
 	/* TRANSLATORS: Label for button in aven’s cavern log window which
 	 * causes the survey data to be reprocessed. */
 	AppendToPage(wxString::Format(wxT("<avenbutton default id=%d name=\"%s\">"),
 				      (int)LOG_REPROCESS,
 				      wmsg(/*Reprocess*/184).c_str()));
-	if (retval == -1) {
-	    wxString m = wxT("Problem running cavern: ");
-	    m += wxString(strerror(errno), wxConvUTF8);
-	    wxGetApp().ReportError(m);
-	    return -2;
-	}
-	return -1;
+	return;
     }
     AppendToPage(wxString::Format(wxT("<avenbutton id=%d name=\"%s\">"),
 				  (int)LOG_REPROCESS,
@@ -470,24 +667,34 @@ abort:
     AppendToPage(wxString::Format(wxT("<avenbutton default id=%d>"), (int)wxID_OK));
     Update();
     init_done = false;
-    return link_count;
+
+    {
+	wxString file3d(filename, 0, filename.length() - 3);
+	file3d.append(wxT("3d"));
+	if (!mainfrm->LoadData(file3d, survey)) {
+	    return;
+	}
+    }
+
+    if (link_count == 0) {
+	wxCommandEvent dummy;
+	OnOK(dummy);
+    }
 }
 
 void
-CavernLogWindow::OnReprocess(wxCommandEvent & e)
+CavernLogWindow::OnEndProcess(wxProcessEvent & evt)
 {
-    SetPage(wxString());
-    int result = process(filename);
-    if (result < 0) return;
-    mainfrm->AddToFileHistory(filename);
-    wxString file3d(filename, 0, filename.length() - 3);
-    file3d.append(wxT("3d"));
-    if (!mainfrm->LoadData(file3d, survey)) {
-	return;
-    }
-    if (result == 0) {
-	OnOK(e);
-    }
+    CavernOutputEvent * e = new CavernOutputEvent();
+    // Zero length indicates successful exit, negative length unsuccessful exit.
+    e->len = (evt.GetExitCode() == 0 ? 0 : -1);
+    QueueEvent(e);
+}
+
+void
+CavernLogWindow::OnReprocess(wxCommandEvent &)
+{
+    process(filename);
 }
 
 void
