@@ -1,6 +1,6 @@
 /* commands.c
  * Code for directives
- * Copyright (C) 1991-2003,2004,2005,2006,2010,2011,2012,2013,2014,2015 Olly Betts
+ * Copyright (C) 1991-2003,2004,2005,2006,2010,2011,2012,2013,2014,2015,2016 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@ default_grade(settings *s)
    /* Values correspond to those in bcra5.svx */
    s->Var[Q_POS] = (real)sqrd(0.05);
    s->Var[Q_LENGTH] = (real)sqrd(0.05);
+   s->Var[Q_BACKLENGTH] = (real)sqrd(0.05);
    s->Var[Q_COUNT] = (real)sqrd(0.05);
    s->Var[Q_DX] = s->Var[Q_DY] = s->Var[Q_DZ] = (real)sqrd(0.05);
    s->Var[Q_BEARING] = (real)sqrd(rad(0.5));
@@ -254,8 +255,8 @@ typedef enum {
    CMD_NULL = -1, CMD_ALIAS, CMD_BEGIN, CMD_CALIBRATE, CMD_CASE, CMD_COPYRIGHT,
    CMD_CS, CMD_DATA, CMD_DATE, CMD_DECLINATION, CMD_DEFAULT, CMD_END,
    CMD_ENTRANCE, CMD_EQUATE, CMD_EXPORT, CMD_FIX, CMD_FLAGS, CMD_INCLUDE,
-   CMD_INFER, CMD_INSTRUMENT, CMD_PREFIX, CMD_REQUIRE, CMD_SD, CMD_SET,
-   CMD_SOLVE, CMD_TEAM, CMD_TITLE, CMD_TRUNCATE, CMD_UNITS
+   CMD_INFER, CMD_INSTRUMENT, CMD_PREFIX, CMD_REF, CMD_REQUIRE, CMD_SD,
+   CMD_SET, CMD_SOLVE, CMD_TEAM, CMD_TITLE, CMD_TRUNCATE, CMD_UNITS
 } cmds;
 
 static const sztok cmd_tab[] = {
@@ -283,6 +284,7 @@ static const sztok cmd_tab[] = {
 #ifndef NO_DEPRECATED
      {"PREFIX",    CMD_PREFIX},
 #endif
+     {"REF",	   CMD_REF},
      {"REQUIRE",   CMD_REQUIRE},
      {"SD",	   CMD_SD},
      {"SET",       CMD_SET},
@@ -383,6 +385,8 @@ get_qlist(unsigned long mask_bad)
 	{"BACKCLINO",    Q_BACKGRADIENT },    /* alternative name */
 	{"BACKCOMPASS",  Q_BACKBEARING },     /* alternative name */
 	{"BACKGRADIENT", Q_BACKGRADIENT },
+	{"BACKLENGTH",   Q_BACKLENGTH },
+	{"BACKTAPE",     Q_BACKLENGTH },    /* alternative name */
 	{"BEARING",      Q_BEARING },
 	{"CEILING",      Q_UP },          /* alternative name */
 	{"CLINO",	 Q_GRADIENT },    /* alternative name */
@@ -1071,8 +1075,11 @@ cmd_export(void)
    fExportUsed = fTrue;
    do {
       int depth = 0;
-      pfx = read_prefix(PFX_STATION);
-      {
+      pfx = read_prefix(PFX_STATION|PFX_NEW);
+      if (pfx == NULL) {
+	 /* The argument was an existing station. */
+	 /* FIXME */
+      } else {
 	 prefix *p = pfx;
 	 while (p != NULL && p != pcs->Prefix) {
 	    depth++;
@@ -1125,6 +1132,8 @@ cmd_data(void)
 	{"BACKCLINO",    BackClino }, /* alternative name */
 	{"BACKCOMPASS",  BackComp }, /* alternative name */
 	{"BACKGRADIENT", BackClino },
+	{"BACKLENGTH",   BackTape },
+	{"BACKTAPE",     BackTape }, /* alternative name */
 	{"BEARING",      Comp },
 	{"CEILING",      Up }, /* alternative name */
 	{"CLINO",	 Clino }, /* alternative name */
@@ -1160,7 +1169,7 @@ cmd_data(void)
    };
 
 #define MASK_stns BIT(Fr) | BIT(To) | BIT(Station)
-#define MASK_tape BIT(Tape) | BIT(FrCount) | BIT(ToCount) | BIT(Count)
+#define MASK_tape BIT(Tape) | BIT(BackTape) | BIT(FrCount) | BIT(ToCount) | BIT(Count)
 #define MASK_dpth BIT(FrDepth) | BIT(ToDepth) | BIT(Depth) | BIT(DepthChange)
 #define MASK_comp BIT(Comp) | BIT(BackComp)
 #define MASK_clin BIT(Clino) | BIT(BackClino)
@@ -1431,7 +1440,7 @@ cmd_data(void)
 
    /* Check the supplied readings form a sufficient set. */
    if (style != STYLE_PASSAGE) {
-       if (mUsed & (BIT(Fr) | BIT(To)))
+       if ((mUsed & (BIT(Fr) | BIT(To))) == (BIT(Fr) | BIT(To)))
 	   mUsed |= BIT(Station);
        else if (TSTBIT(mUsed, Station))
 	   mUsed |= BIT(Fr) | BIT(To);
@@ -1443,19 +1452,15 @@ cmd_data(void)
    if (mUsed & (BIT(Clino) | BIT(BackClino)))
       mUsed |= BIT(Clino) | BIT(BackClino);
 
-   if (mUsed & (BIT(FrDepth) | BIT(ToDepth)))
+   if ((mUsed & (BIT(FrDepth) | BIT(ToDepth))) == (BIT(FrDepth) | BIT(ToDepth)))
       mUsed |= BIT(Depth) | BIT(DepthChange);
-   else if (TSTBIT(mUsed, Depth))
-      mUsed |= BIT(FrDepth) | BIT(ToDepth) | BIT(DepthChange);
-   else if (TSTBIT(mUsed, DepthChange))
-      mUsed |= BIT(FrDepth) | BIT(ToDepth) | BIT(Depth);
+   else if (mUsed & (BIT(Depth) | BIT(DepthChange)))
+      mUsed |= BIT(FrDepth) | BIT(ToDepth) | BIT(Depth) | BIT(DepthChange);
 
-   if (mUsed & (BIT(FrCount) | BIT(ToCount)))
-      mUsed |= BIT(Count) | BIT(Tape);
-   else if (TSTBIT(mUsed, Count))
-      mUsed |= BIT(FrCount) | BIT(ToCount) | BIT(Tape);
-   else if (TSTBIT(mUsed, Tape))
-      mUsed |= BIT(FrCount) | BIT(ToCount) | BIT(Count);
+   if ((mUsed & (BIT(FrCount) | BIT(ToCount))) == (BIT(FrCount) | BIT(ToCount)))
+      mUsed |= BIT(Count) | BIT(Tape) | BIT(BackTape);
+   else if (mUsed & (BIT(Count) | BIT(Tape) | BIT(BackTape)))
+      mUsed |= BIT(FrCount) | BIT(ToCount) | BIT(Count) | BIT(Tape) | BIT(BackTape);
 
 #if 0
    printf("mUsed = 0x%x, opt = 0x%x, mask = 0x%x\n", mUsed,
@@ -2151,6 +2156,16 @@ cmd_truncate(void)
    pcs->Truncate = (truncate_at == 0) ? INT_MAX : truncate_at;
 }
 
+static void
+cmd_ref(void)
+{
+   /* Just syntax check for now. */
+   char *ref = NULL;
+   int ref_len;
+   read_string(&ref, &ref_len);
+   s_free(&ref);
+}
+
 static void
 cmd_require(void)
 {
@@ -2293,6 +2308,7 @@ static const cmd_fn cmd_funcs[] = {
 #ifndef NO_DEPRECATED
    cmd_prefix,
 #endif
+   cmd_ref,
    cmd_require,
    cmd_sd,
    cmd_set,
@@ -2328,11 +2344,26 @@ handle_command(void)
 	  * *end fred */
 	 compile_error(/**EXPORT must immediately follow “*BEGIN <SURVEY>”*/57);
       break;
+    case CMD_ALIAS:
+    case CMD_CALIBRATE:
+    case CMD_CASE:
     case CMD_COPYRIGHT:
+    case CMD_CS:
+    case CMD_DATA:
     case CMD_DATE:
+    case CMD_DECLINATION:
+    case CMD_DEFAULT:
+    case CMD_FLAGS:
+    case CMD_INFER:
     case CMD_INSTRUMENT:
+    case CMD_REF:
+    case CMD_REQUIRE:
+    case CMD_SD:
+    case CMD_SET:
     case CMD_TEAM:
     case CMD_TITLE:
+    case CMD_TRUNCATE:
+    case CMD_UNITS:
       /* These can occur between *begin and *export */
       break;
     default:
