@@ -1,6 +1,6 @@
 /* datain.c
  * Reads in survey files, dealing with special characters, keywords & data
- * Copyright (C) 1991-2003,2005,2009,2010,2011,2012,2013,2014,2015 Olly Betts
+ * Copyright (C) 1991-2003,2005,2009,2010,2011,2012,2013,2014,2015,2016 Olly Betts
  * Copyright (C) 2004 Simeon Warner
  *
  * This program is free software; you can redistribute it and/or modify
@@ -254,22 +254,9 @@ skipline(void)
    while (!isEol(ch)) nextch();
 }
 
-#ifndef NO_PERCENTAGE
-static long int filelen;
-#endif
-
 static void
 process_bol(void)
 {
-#ifndef NO_PERCENTAGE
-   /* print %age of file done */
-   if (filelen > 0) {
-      filepos fp;
-      get_pos(&fp);
-      printf("%d%%\r", (int)(100 * fp.offset / filelen));
-   }
-#endif
-
    nextch();
    skipblanks();
 }
@@ -297,6 +284,7 @@ process_eol(void)
 	 push_back(ch);
 	 break;
       }
+      if (ch == '\n') eolchar = ch;
    }
    file.lpos = ftell(file.fh);
 }
@@ -325,6 +313,7 @@ read_reading(reading r, bool f_optional)
    q_quantity q;
    switch (r) {
       case Tape: q = Q_LENGTH; break;
+      case BackTape: q = Q_BACKLENGTH; break;
       case Comp: q = Q_BEARING; break;
       case BackComp: q = Q_BACKBEARING; break;
       case Clino: q = Q_GRADIENT; break;
@@ -419,27 +408,11 @@ data_file(const char *pth, const char *fnm)
       file.reported_where = fFalse;
    }
 
-   if (fPercent) printf("%s:\n", fnm);
-
    using_data_file(file.filename);
 
    begin_lineno_store = pcs->begin_lineno;
    pcs->begin_lineno = 0;
 
-#ifndef NO_PERCENTAGE
-   /* Try to find how long the file is...
-    * However, under ANSI fseek( ..., SEEK_END) may not be supported */
-   filelen = 0;
-   if (fPercent) {
-      if (fseek(file.fh, 0l, SEEK_END) == 0) {
-	 filepos fp;
-	 get_pos(&fp);
-	 filelen = fp.offset;
-      }
-      rewind(file.fh); /* reset file ptr to start & clear any error state */
-   }
-#endif
-
    if (fmt == FMT_DAT) {
       short *t;
       int i;
@@ -750,10 +723,6 @@ data_file(const char *pth, const char *fnm)
     * including file */
    f_export_ok = fFalse;
 
-#ifndef NO_PERCENTAGE
-   if (fPercent) putnl();
-#endif
-
    if (pcs->begin_lineno) {
       error_in_file(file.filename, pcs->begin_lineno,
 		    /*BEGIN with no matching END in this file*/23);
@@ -926,7 +895,7 @@ handle_compass(real *p_var)
 	 real diff = comp - backcomp;
 	 real adj = fabs(diff) > M_PI ? M_PI : 0;
 	 diff -= floor((diff + M_PI) / (2 * M_PI)) * 2 * M_PI;
-	 if (sqrd(diff / 3.0) > compvar + VAR(Q_BACKBEARING)) {
+	 if (sqrd(diff / 3.0) > compvar + VAR(BackComp)) {
 	    /* fore and back readings differ by more than 3 sds */
 	    /* TRANSLATORS: %s is replaced by the amount the readings disagree
 	     * by, e.g. "2.5°" or "3ᵍ". */
@@ -1071,8 +1040,9 @@ process_normal(prefix *fr, prefix *to, bool fToFirst,
       dx = dy = (real)0.0;
       if (ctype != CTYPE_OMIT) {
 	 if (backctype != CTYPE_OMIT && (clin > 0) == (backclin > 0)) {
-	    /* We've got two UPs or two DOWNs - FIXME: not ideal message */
-	    compile_error_skip(/*CLINO and BACKCLINO readings must be of the same type*/84);
+	    /* TRANSLATORS: We've been told the foresight and backsight are
+	     * both "UP", or that they're both "DOWN". */
+	    compile_error_skip(/*Plumbed CLINO and BACKCLINO readings can't be in the same direction*/92);
 	    return 0;
 	 }
 	 dz = (clin > (real)0.0) ? tape : -tape;
@@ -1488,7 +1458,7 @@ data_normal(void)
 
    reading *ordering;
 
-   VAL(Tape) = 0;
+   VAL(Tape) = VAL(BackTape) = HUGE_REAL;
    VAL(Comp) = VAL(BackComp) = HUGE_REAL;
    VAL(FrCount) = VAL(ToCount) = 0;
    VAL(FrDepth) = VAL(ToDepth) = 0;
@@ -1550,9 +1520,9 @@ data_normal(void)
 	  }
 	  break;
        }
-       case Tape:
-	  read_reading(Tape, fFalse);
-	  if (VAL(Tape) < (real)0.0)
+       case Tape: case BackTape:
+	  read_reading(*ordering, fFalse);
+	  if (VAL(*ordering) < (real)0.0)
 	     compile_warning(-/*Negative tape reading*/60);
 	  break;
        case Count:
@@ -1680,7 +1650,9 @@ data_normal(void)
 	      * (frdepth always zero, todepth is change of depth) and also
 	      * works for STYLE_NORMAL (both remain 0) */
 	     if (TSTBIT(pcs->infer, INFER_EQUATES) &&
-		 VAL(Tape) == (real)0.0 && VAL(FrDepth) == VAL(ToDepth)) {
+		 (VAL(Tape) == (real)0.0 || VAL(Tape) == HUGE_REAL) &&
+		 (VAL(BackTape) == (real)0.0 || VAL(BackTape) == HUGE_REAL) &&
+		 VAL(FrDepth) == VAL(ToDepth)) {
 		process_equate(fr, to);
 		goto inferred_equate;
 	     }
@@ -1691,11 +1663,32 @@ data_normal(void)
 	     }
 	     if (fTopofil) {
 		VAL(Tape) *= pcs->units[Q_COUNT] * pcs->sc[Q_COUNT];
-	     } else {
+	     } else if (VAL(Tape) != HUGE_REAL) {
 		VAL(Tape) *= pcs->units[Q_LENGTH];
 		VAL(Tape) -= pcs->z[Q_LENGTH];
 		VAL(Tape) *= pcs->sc[Q_LENGTH];
 	     }
+	     if (VAL(BackTape) != HUGE_REAL) {
+		VAL(BackTape) *= pcs->units[Q_BACKLENGTH];
+		VAL(BackTape) -= pcs->z[Q_BACKLENGTH];
+		VAL(BackTape) *= pcs->sc[Q_BACKLENGTH];
+		if (VAL(Tape) != HUGE_REAL) {
+		   real diff = VAL(Tape) - VAL(BackTape);
+		   if (sqrd(diff / 3.0) > VAR(Tape) + VAR(BackTape)) {
+		      /* fore and back readings differ by more than 3 sds */
+		      /* TRANSLATORS: %s is replaced by the amount the readings disagree
+		       * by, e.g. "0.12m" or "0.2ft". */
+		      warn_readings_differ(/*TAPE reading and BACKTAPE reading disagree by %s*/97,
+					   diff, get_length_units(Q_LENGTH));
+		   }
+		   VAL(Tape) = VAL(Tape) / VAR(Tape) + VAL(BackTape) / VAR(BackTape);
+		   VAR(Tape) = (VAR(Tape) + VAR(BackTape)) / 4;
+		   VAL(Tape) *= VAR(Tape);
+		} else {
+		   VAL(Tape) = VAL(BackTape);
+		   VAR(Tape) = VAR(BackTape);
+		}
+	     }
 	     implicit_splay = TSTBIT(pcs->flags, FLAGS_IMPLICIT_SPLAY);
 	     pcs->flags &= ~(BIT(FLAGS_ANON_ONE_END) | BIT(FLAGS_IMPLICIT_SPLAY));
 	     save_flags = pcs->flags;
@@ -1774,18 +1767,41 @@ data_normal(void)
 	      * (frdepth always zero, todepth is change of depth) and also
 	      * works for STYLE_NORMAL (both remain 0) */
 	     if (TSTBIT(pcs->infer, INFER_EQUATES) &&
-		 VAL(Tape) == (real)0.0 && VAL(FrDepth) == VAL(ToDepth)) {
+		 (VAL(Tape) == (real)0.0 || VAL(Tape) == HUGE_REAL) &&
+		 (VAL(BackTape) == (real)0.0 || VAL(BackTape) == HUGE_REAL) &&
+		 VAL(FrDepth) == VAL(ToDepth)) {
 		process_equate(fr, to);
 		process_eol();
 		return;
 	     }
 	     if (fTopofil) {
 		VAL(Tape) *= pcs->units[Q_COUNT] * pcs->sc[Q_COUNT];
-	     } else {
+	     } else if (VAL(Tape) != HUGE_REAL) {
 		VAL(Tape) *= pcs->units[Q_LENGTH];
 		VAL(Tape) -= pcs->z[Q_LENGTH];
 		VAL(Tape) *= pcs->sc[Q_LENGTH];
 	     }
+	     if (VAL(BackTape) != HUGE_REAL) {
+		VAL(BackTape) *= pcs->units[Q_BACKLENGTH];
+		VAL(BackTape) -= pcs->z[Q_BACKLENGTH];
+		VAL(BackTape) *= pcs->sc[Q_BACKLENGTH];
+		if (VAL(Tape) != HUGE_REAL) {
+		   real diff = VAL(Tape) - VAL(BackTape);
+		   if (sqrd(diff / 3.0) > VAR(Tape) + VAR(BackTape)) {
+		      /* fore and back readings differ by more than 3 sds */
+		      /* TRANSLATORS: %s is replaced by the amount the readings disagree
+		       * by, e.g. "0.12m" or "0.2ft". */
+		      warn_readings_differ(/*TAPE reading and BACKTAPE reading disagree by %s*/97,
+					   diff, get_length_units(Q_LENGTH));
+		   }
+		   VAL(Tape) = VAL(Tape) / VAR(Tape) + VAL(BackTape) / VAR(BackTape);
+		   VAR(Tape) = (VAR(Tape) + VAR(BackTape)) / 4;
+		   VAL(Tape) *= VAR(Tape);
+		} else {
+		   VAL(Tape) = VAL(BackTape);
+		   VAR(Tape) = VAR(BackTape);
+		}
+	     }
 	     implicit_splay = TSTBIT(pcs->flags, FLAGS_IMPLICIT_SPLAY);
 	     pcs->flags &= ~(BIT(FLAGS_ANON_ONE_END) | BIT(FLAGS_IMPLICIT_SPLAY));
 	     save_flags = pcs->flags;
