@@ -1,6 +1,6 @@
 /* extend.c
  * Produce an extended elevation
- * Copyright (C) 1995-2002,2005,2010,2011,2013,2014 Olly Betts
+ * Copyright (C) 1995-2002,2005,2010,2011,2013,2014,2016 Olly Betts
  * Copyright (C) 2004,2005 John Pybus
  *
  * This program is free software; you can redistribute it and/or modify
@@ -47,6 +47,7 @@ typedef struct stn {
 
 typedef struct POINT {
    img_point p;
+   double X;
    const stn *stns;
    unsigned int order;
    char dir;
@@ -74,12 +75,14 @@ typedef struct LEG {
 #define ERIGHT 0x02
 #define ESWAP  0x04
 
-static point headpoint = {{0, 0, 0}, NULL, 0, 0, 0, 0, NULL};
+static point headpoint = {{0, 0, 0}, 0, NULL, 0, 0, 0, 0, NULL};
 
 static leg headleg = {NULL, NULL, NULL, 0, 0, 0, 0, NULL};
 
 static img *pimg_out;
 
+static int show_breaks = 0;
+
 static void do_stn(point *, double, const char *, int, int);
 
 typedef struct pfx {
@@ -124,6 +127,7 @@ find_point(const img_point *pt)
 
    p = osmalloc(ossizeof(point));
    p->p = *pt;
+   p->X = HUGE_VAL;
    p->stns = NULL;
    p->order = 0;
    p->dir = 0;
@@ -266,7 +270,8 @@ parseconfigline(const char *fnm, char *ln)
       if (*ll == 0)
 	 fatalerror_in_file(fnm, lineno, /*Expecting station name*/28);
       ln = delimword(lc, &lc);
-      if (*ln == 0) { /* One argument, look for point to switch at. */
+      if (*ln == 0) {
+	 /* One argument - look for station to switch at. */
 	 for (p = headpoint.next; p != NULL; p = p->next) {
 	    for (s = p->stns; s; s = s->next) {
 	       if (strcmp(s->label, ll)==0) {
@@ -279,7 +284,8 @@ parseconfigline(const char *fnm, char *ln)
 	    }
 	 }
 	 warning_in_file(fnm, lineno, /*Failed to find station %s*/510, ll);
-      } else { /* Two arguments look for a specific leg */
+      } else {
+	 /* Two arguments - look for a specified leg. */
 	 for (l = headleg.next; l; l=l->next) {
 	    point * fr = l->fr;
 	    point * to = l->to;
@@ -311,7 +317,8 @@ parseconfigline(const char *fnm, char *ln)
       if (*ll == 0)
 	 fatalerror_in_file(fnm, lineno, /*Expecting station name*/28);
       ln = delimword(lc, &lc);
-      if (*ln == 0) { /* One argument, look for point to switch at. */
+      if (*ln == 0) {
+	 /* One argument - look for station to switch at. */
 	 for (p = headpoint.next; p != NULL; p = p->next) {
 	    for (s = p->stns; s; s = s->next) {
 	       if (strcmp(s->label, ll)==0) {
@@ -324,7 +331,8 @@ parseconfigline(const char *fnm, char *ln)
 	    }
 	 }
 	 warning_in_file(fnm, lineno, /*Failed to find station %s*/510, ll);
-      } else { /* Two arguments look for a specific leg */
+      } else {
+	 /* Two arguments - look for a specified leg. */
 	 for (l = headleg.next; l; l=l->next) {
 	    point * fr = l->fr;
 	    point * to = l->to;
@@ -353,7 +361,8 @@ parseconfigline(const char *fnm, char *ln)
       if (*ll == 0)
 	 fatalerror_in_file(fnm, lineno, /*Expecting station name*/28);
       ln = delimword(lc, &lc);
-      if (*ln == 0) { /* One argument, look for point to switch at. */
+      if (*ln == 0) {
+	 /* One argument - look for station to switch at. */
 	 for (p = headpoint.next; p != NULL; p = p->next) {
 	    for (s = p->stns; s; s = s->next) {
 	       if (strcmp(s->label, ll)==0) {
@@ -366,7 +375,8 @@ parseconfigline(const char *fnm, char *ln)
 	    }
 	 }
 	 warning_in_file(fnm, lineno, /*Failed to find station %s*/510, ll);
-      } else { /* Two arguments look for a specific leg */
+      } else {
+	 /* Two arguments - look for a specified leg. */
 	 for (l = headleg.next; l; l=l->next) {
 	    point * fr = l->fr;
 	    point * to = l->to;
@@ -395,7 +405,8 @@ parseconfigline(const char *fnm, char *ln)
       if (*ll == 0)
 	 fatalerror_in_file(fnm, lineno, /*Expecting station name*/28);
       ln = delimword(lc, &lc);
-      if (*ln == 0) { /* One argument, look for point to break at. */
+      if (*ln == 0) {
+	 /* One argument - look for specified station to break at. */
 	 for (p = headpoint.next; p != NULL; p = p->next) {
 	    for (s = p->stns; s; s = s->next) {
 	       if (strcmp(s->label, ll)==0) {
@@ -408,7 +419,9 @@ parseconfigline(const char *fnm, char *ln)
 	    }
 	 }
 	 warning_in_file(fnm, lineno, /*Failed to find station %s*/510, ll);
-      } else { /* Two arguments look for a specific leg */
+      } else {
+	 /* Two arguments - look for specified leg and disconnect it at the
+	  * first station. */
 	 for (l = headleg.next; l; l=l->next) {
 	    point * fr = l->fr;
 	    point * to = l->to;
@@ -447,16 +460,21 @@ static const struct option long_opts[] = {
    /* const char *name; int has_arg (0 no_argument, 1 required_*, 2 optional_*); int *flag; int val; */
    {"survey", required_argument, 0, 's'},
    {"specfile", required_argument, 0, 'p'},
+   {"show-breaks", no_argument, 0, 'b' },
    {"help", no_argument, 0, HLP_HELP},
    {"version", no_argument, 0, HLP_VERSION},
    {0, 0, 0, 0}
 };
 
-#define short_opts "s:p:"
+#define short_opts "s:p:b"
 
 static struct help_msg help[] = {
 /*				<-- */
    {HLP_ENCODELONG(0),        /*only load the sub-survey with this prefix*/199, 0},
+   /* TRANSLATORS: --help output for extend --specfile option */
+   {HLP_ENCODELONG(1),        /*.espec file to control extending*/90, 0},
+   /* TRANSLATORS: --help output for extend --show-breaks option */
+   {HLP_ENCODELONG(2),        /*show breaks with surface survey legs in output*/91, 0},
    {0, 0, 0}
 };
 
@@ -483,8 +501,17 @@ main(int argc, char **argv)
    while (1) {
       int opt = cmdline_getopt();
       if (opt == EOF) break;
-      if (opt == 's') survey = optarg;
-      if (opt == 'p') specfile = optarg;
+      switch (opt) {
+	 case 'b':
+	    show_breaks = 1;
+	    break;
+	 case 's':
+	    survey = optarg;
+	    break;
+	 case 'p':
+	    specfile = optarg;
+	    break;
+      }
    }
    fnm_in = argv[optind++];
    if (argv[optind]) {
@@ -563,9 +590,11 @@ main(int argc, char **argv)
       osfree(fnm_used);
    }
 
-   if (start == NULL) { /* i.e. start wasn't specified in specfile */
-
-      /* start at the highest entrance with some legs attached */
+   if (start == NULL) {
+      /* *start wasn't specified in specfile.
+       *
+       * Start at the highest entrance with some legs attached.
+       */
       for (p = headpoint.next; p != NULL; p = p->next) {
 	 if (p->order > 0 && p->p.z > zMax) {
 	    const stn *s;
@@ -579,7 +608,7 @@ main(int argc, char **argv)
 	 }
       }
       if (start == NULL) {
-	 /* if no entrances with legs, start at the highest 1-node */
+	 /* If no entrances with legs, start at the highest 1-node. */
 	 for (p = headpoint.next; p != NULL; p = p->next) {
 	    if (p->order == 1 && p->p.z > zMax) {
 	       start = p;
@@ -671,97 +700,68 @@ do_stn(point *p, double X, const char *prefix, int dir, int labOnly)
    double dX;
    const stn *s;
    int odir = dir;
+   int try_all;
 
    for (s = p->stns; s; s = s->next) {
       img_write_item(pimg_out, img_LABEL, s->flags, s->label, X, 0, p->p.z);
    }
+   if (show_breaks && p->X != HUGE_VAL && p->X != X) {
+      /* Draw "surface" leg between broken stations. */
+      img_write_item(pimg_out, img_MOVE, 0, NULL, p->X, 0, p->p.z);
+      img_write_item(pimg_out, img_LINE, img_FLAG_SURFACE, NULL, X, 0, p->p.z);
+   }
+   p->X = X;
    if (labOnly || p->fBroken) {
       return;
    }
 
-   lp = &headleg;
-   for (l = lp->next; l; lp = l, l = lp->next) {
-      dir = odir;
-      if (l->fDone) {
-	 /* this case happens if a recursive call causes the next leg to be
-	  * removed, leaving our next pointing to a leg which has been dealt
-	  * with... */
-	 continue;
-      }
-      if (l->prefix != prefix) {
-	 continue;
-      }
-      if (l->to == p) {
-	 if (l->broken & BREAK_TO) continue;
-	 lp->next = l->next;
-	 /* adjust direction of extension if necessary */
-	 dir = adjust_direction(dir, l->to->dir);
-	 dir = adjust_direction(dir, l->dir);
+   /* It's better to follow legs along a survey, so make two passes and only
+    * follow legs in the same survey for the first pass.
+    */
+   for (try_all = 0; try_all != 2; ++try_all) {
+      lp = &headleg;
+      for (l = lp->next; l; lp = l, l = lp->next) {
+	 dir = odir;
+	 if (l->fDone) {
+	    /* this case happens iff a recursive call causes the next leg to be
+	     * removed, leaving our next pointing to a leg which has been dealt
+	     * with... */
+	    continue;
+	 }
+	 if (!try_all && l->prefix != prefix) {
+	    continue;
+	 }
+	 if (l->to == p) {
+	    if (l->broken & BREAK_TO) continue;
+	    lp->next = l->next;
+	    /* adjust direction of extension if necessary */
+	    dir = adjust_direction(dir, l->to->dir);
+	    dir = adjust_direction(dir, l->dir);
 
-	 dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
-	 if (dir == ELEFT) dX = -dX;
-	 img_write_item(pimg_out, img_MOVE, 0, NULL, X + dX, 0, l->fr->p.z);
-	 img_write_item(pimg_out, img_LINE, l->flags, l->prefix,
-			X, 0, l->to->p.z);
-	 l->fDone = 1;
-	 do_stn(l->fr, X + dX, l->prefix, dir, (l->broken & BREAK_FR));
-	 l = lp;
-      } else if (l->fr == p) {
-	 if (l->broken & BREAK_FR) continue;
-	 lp->next = l->next;
-	 /* adjust direction of extension if necessary */
-	 dir = adjust_direction(dir, l->fr->dir);
-	 dir = adjust_direction(dir, l->dir);
+	    dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
+	    if (dir == ELEFT) dX = -dX;
+	    img_write_item(pimg_out, img_MOVE, 0, NULL, X + dX, 0, l->fr->p.z);
+	    img_write_item(pimg_out, img_LINE, l->flags, l->prefix,
+			   X, 0, l->to->p.z);
+	    l->fDone = 1;
+	    do_stn(l->fr, X + dX, l->prefix, dir, (l->broken & BREAK_FR));
+	    l = lp;
+	 } else if (l->fr == p) {
+	    if (l->broken & BREAK_FR) continue;
+	    lp->next = l->next;
+	    /* adjust direction of extension if necessary */
+	    dir = adjust_direction(dir, l->fr->dir);
+	    dir = adjust_direction(dir, l->dir);
 
-	 dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
-	 if (dir == ELEFT) dX = -dX;
-	 img_write_item(pimg_out, img_MOVE, 0, NULL, X, 0, l->fr->p.z);
-	 img_write_item(pimg_out, img_LINE, l->flags, l->prefix,
-			X + dX, 0, l->to->p.z);
-	 l->fDone = 1;
-	 do_stn(l->to, X + dX, l->prefix, dir, (l->broken & BREAK_TO));
-	 l = lp;
-      }
-   }
-   lp = &headleg;
-   for (l = lp->next; l; lp = l, l = lp->next) {
-      dir = odir;
-      if (l->fDone) {
-	 /* this case happens iff a recursive call causes the next leg to be
-	  * removed, leaving our next pointing to a leg which has been dealt
-	  * with... */
-	 continue;
-      }
-      if (l->to == p) {
-	 if (l->broken & BREAK_TO) continue;
-	 lp->next = l->next;
-	 /* adjust direction of extension if necessary */
-	 dir = adjust_direction(dir, l->to->dir);
-	 dir = adjust_direction(dir, l->dir);
-
-	 dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
-	 if (dir == ELEFT) dX = -dX;
-	 img_write_item(pimg_out, img_MOVE, 0, NULL, X + dX, 0, l->fr->p.z);
-	 img_write_item(pimg_out, img_LINE, l->flags, l->prefix,
-			X, 0, l->to->p.z);
-	 l->fDone = 1;
-	 do_stn(l->fr, X + dX, l->prefix, dir, (l->broken & BREAK_FR));
-	 l = lp;
-      } else if (l->fr == p) {
-	 if (l->broken & BREAK_FR) continue;
-	 lp->next = l->next;
-	 /* adjust direction of extension if necessary */
-	 dir = adjust_direction(dir, l->fr->dir);
-	 dir = adjust_direction(dir, l->dir);
-
-	 dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
-	 if (dir == ELEFT) dX = -dX;
-	 img_write_item(pimg_out, img_MOVE, 0, NULL, X, 0, l->fr->p.z);
-	 img_write_item(pimg_out, img_LINE, l->flags, l->prefix,
-			X + dX, 0, l->to->p.z);
-	 l->fDone = 1;
-	 do_stn(l->to, X + dX, l->prefix, dir, (l->broken & BREAK_TO));
-	 l = lp;
+	    dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
+	    if (dir == ELEFT) dX = -dX;
+	    img_write_item(pimg_out, img_MOVE, 0, NULL, X, 0, l->fr->p.z);
+	    img_write_item(pimg_out, img_LINE, l->flags, l->prefix,
+			   X + dX, 0, l->to->p.z);
+	    l->fDone = 1;
+	    do_stn(l->to, X + dX, l->prefix, dir, (l->broken & BREAK_TO));
+	    l = lp;
+	 }
       }
    }
 }
