@@ -115,6 +115,7 @@ const int MAX_FRAMERATE = 50;
 
 class GfxCore : public GLACanvas {
     Double m_Scale;
+    Double initial_scale;
     int m_ScaleBarWidth;
 
     typedef enum {
@@ -554,7 +555,7 @@ public:
     void MoveViewer(double forward, double up, double right);
 
     void (GfxCore::* AddQuad)(const Vector3 &a, const Vector3 &b,
-                              const Vector3 &c, const Vector3 &d);
+			      const Vector3 &c, const Vector3 &d);
     void (GfxCore::* AddPoly)(const traverse & centreline);
 
     PresentationMark GetView() const;
