@@ -168,6 +168,7 @@ public:
     void SetIndicatorTransform();
 
     void DrawList(unsigned int l);
+    void DrawListZPrepass(unsigned int l);
     void DrawList2D(unsigned int l, glaCoord x, glaCoord y, Double rotation);
     void InvalidateList(unsigned int l) {
 	if (l < drawing_lists.size()) {
@@ -186,7 +187,7 @@ public:
 
     void DrawText(glaCoord x, glaCoord y, glaCoord z, const wxString& str);
     void DrawIndicatorText(int x, int y, const wxString& str);
-    void GetTextExtent(const wxString& str, int * x_ext, int * y_ext);
+    void GetTextExtent(const wxString& str, int * x_ext, int * y_ext) const;
 
     void BeginQuadrilaterals();
     void EndQuadrilaterals();
