@@ -2,7 +2,7 @@
  * Export from Aven as GPX.
  */
 /* Copyright (C) 2012 Olaf Kähler
- * Copyright (C) 2012,2013,2014 Olly Betts
+ * Copyright (C) 2012,2013,2014,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -94,11 +94,12 @@ GPX::passes() const
 }
 
 /* Initialise GPX routines. */
-void GPX::header(const char * title, const char *, time_t datestamp_numeric)
+void GPX::header(const char * title, const char *, time_t datestamp_numeric,
+		 double, double, double, double, double, double)
 {
     fputs(
 "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
-"<gpx version=\"1.0\" creator=\""PACKAGE_STRING" (aven) - http://survex.com/\""
+"<gpx version=\"1.0\" creator=\"" PACKAGE_STRING " (aven) - http://survex.com/\""
 " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
 " xmlns=\"http://www.topografix.com/GPX/1/0\""
 " xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0"
