@@ -63,7 +63,7 @@ html_escape(FILE *fh, const char *s)
 }
 
 KML::KML(const char * input_datum)
-    : pj_input(NULL), pj_output(NULL), in_trkseg(false)
+    : pj_input(NULL), pj_output(NULL), in_linestring(false)
 {
     if (!(pj_input = pj_init_plus(input_datum))) {
 	wxString m = wmsg(/*Failed to initialise input coordinate system “%s”*/287);
@@ -88,8 +88,7 @@ KML::~KML()
 const int *
 KML::passes() const
 {
-    // static const int default_passes[] = { LABELS|ENTS|FIXES|EXPORTS, LEGS|SURF, 0 };
-    static const int default_passes[] = { LABELS|ENTS|FIXES|EXPORTS, 0 };
+    static const int default_passes[] = { LABELS|ENTS|FIXES|EXPORTS, LEGS|SURF, 0 };
     return default_passes;
 }
 
@@ -103,6 +102,16 @@ void KML::header(const char * title, const char *, time_t,
     fputs("<Document><name>", fh);
     html_escape(fh, title);
     fputs("</name>\n", fh);
+    // Set up styles for the icons to reduce the file size.
+    fputs("<Style id=\"fix\"><IconStyle>"
+	  "<Icon><href>http://maps.google.com/mapfiles/kml/paddle/red-blank.png</href></Icon>"
+	  "</IconStyle></Style>\n", fh);
+    fputs("<Style id=\"exp\"><IconStyle>"
+	  "<Icon><href>http://maps.google.com/mapfiles/kml/paddle/blu-blank.png</href></Icon>"
+	  "</IconStyle></Style>\n", fh);
+    fputs("<Style id=\"ent\"><IconStyle>"
+	  "<Icon><href>http://maps.google.com/mapfiles/kml/paddle/grn-blank.png</href></Icon>"
+	  "</IconStyle></Style>\n", fh);
     // FIXME: does KML allow bounds?
     // NB Lat+long bounds are not necessarily the same as the bounds in survex
     // coords translated to WGS84 lat+long...
@@ -111,37 +120,32 @@ void KML::header(const char * title, const char *, time_t,
 void
 KML::line(const img_point *p1, const img_point *p, bool /*fSurface*/, bool fPendingMove)
 {
-    (void)p1;
-    (void)p;
-    (void)fPendingMove;
-#if 0
     if (fPendingMove) {
-	if (in_trkseg) {
-	    fputs("</trkseg><trkseg>\n", fh);
+	if (!in_linestring) {
+	    in_linestring = true;
+	    fputs("<Placemark><MultiGeometry>\n", fh);
 	} else {
-	    fputs("<trk><trkseg>\n", fh);
-	    in_trkseg = true;
+	    fputs("</coordinates></LineString>\n", fh);
 	}
+	fputs("<LineString><altitudeMode>absolute</altitudeMode><coordinates>\n", fh);
 	double X = p1->x, Y = p1->y, Z = p1->z;
 	pj_transform(pj_input, pj_output, 1, 1, &X, &Y, &Z);
 	X = deg(X);
 	Y = deg(Y);
 	// %.8f is at worst just over 1mm.
-	fprintf(fh, "<trkpt lon=\"%.8f\" lat=\"%.8f\"><ele>%.2f</ele></trkpt>\n", X, Y, Z);
+	fprintf(fh, "%.8f,%.8f,%.2f\n", X, Y, Z);
     }
     double X = p->x, Y = p->y, Z = p->z;
     pj_transform(pj_input, pj_output, 1, 1, &X, &Y, &Z);
     X = deg(X);
     Y = deg(Y);
     // %.8f is at worst just over 1mm.
-    fprintf(fh, "<trkpt lon=\"%.8f\" lat=\"%.8f\"><ele>%.2f</ele></trkpt>\n", X, Y, Z);
-#endif
+    fprintf(fh, "%.8f,%.8f,%.2f\n", X, Y, Z);
 }
 
 void
 KML::label(const img_point *p, const char *s, bool /*fSurface*/, int type)
 {
-    (void)type;
     double X = p->x, Y = p->y, Z = p->z;
     pj_transform(pj_input, pj_output, 1, 1, &X, &Y, &Z);
     X = deg(X);
@@ -150,29 +154,25 @@ KML::label(const img_point *p, const char *s, bool /*fSurface*/, int type)
     fprintf(fh, "<Placemark><Point><coordinates>%.8f,%.8f,%.2f</coordinates></Point><name>", X, Y, Z);
     html_escape(fh, s);
     fputs("</name>", fh);
-#if 0
     // Add a "pin" symbol with colour matching what aven shows.
     switch (type) {
 	case FIXES:
-	    fputs("<sym>Pin, Red</sym>", fh);
+	    fputs("<styleUrl>#fix</styleUrl>", fh);
 	    break;
 	case EXPORTS:
-	    fputs("<sym>Pin, Blue</sym>", fh);
+	    fputs("<styleUrl>#exp</styleUrl>", fh);
 	    break;
 	case ENTS:
-	    fputs("<sym>Pin, Green</sym>", fh);
+	    fputs("<styleUrl>#ent</styleUrl>", fh);
 	    break;
     }
-#endif
     fputs("</Placemark>\n", fh);
 }
 
 void
 KML::footer()
 {
-#if 0
-    if (in_trkseg)
-	fputs("</trkseg></trk>\n", fh);
-#endif
+    if (in_linestring)
+	fputs("</coordinates></LineString></MultiGeometry></Placemark>\n", fh);
     fputs("</Document></kml>\n", fh);
 }
