@@ -39,6 +39,7 @@
 #include "useful.h"
 
 #include <wx/confbase.h>
+//#include <wx/filefn.h>
 #include <wx/filename.h>
 #include <wx/image.h>
 #include <wx/imaglist.h>
@@ -906,6 +907,9 @@ void MainFrm::CreateMenuBar()
     wxString wxmac_fullscreen = wmsg(/*Full Screen &Mode\tF11*/356);
     wxmac_fullscreen.Replace(wxT("\tF11"), wxT("\tRawCtrl+Ctrl+F"), false);
     viewmenu->AppendCheckItem(menu_VIEW_FULLSCREEN, wxmac_fullscreen);
+    // FIXME: On OS X, the standard wording here is "Enter Full Screen" and
+    // "Exit Full Screen", depending whether we are in full screen mode or not,
+    // and this isn't a checked menu item.
 #else
     viewmenu->AppendCheckItem(menu_VIEW_FULLSCREEN, wmsg(/*Full Screen &Mode\tF11*/356));
 #endif
@@ -1106,7 +1110,7 @@ bool MainFrm::LoadData(const wxString& file, const wxString & prefix)
 #endif
 
     // Load the processed survey data.
-    img* survey = img_open_survey(file.mb_str(), prefix.mb_str());
+    img* survey = img_open_survey(file.utf8_str(), prefix.utf8_str());
     if (!survey) {
 	wxString m = wxString::Format(wmsg(img_error2msg(img_error())), file.c_str());
 	wxGetApp().ReportError(m);
@@ -1807,26 +1811,16 @@ void MainFrm::OpenFile(const wxString& file, const wxString& survey)
 	    CavernLogWindow * log = new CavernLogWindow(this, survey, m_Splitter);
 	    wxWindow * win = m_Splitter->GetWindow1();
 	    m_Splitter->ReplaceWindow(win, log);
+	    win->Show(false);
 	    if (m_Splitter->GetWindow2() == NULL) {
 		if (win != m_Gfx) win->Destroy();
 	    } else {
 		if (m_Splitter->IsSplit()) m_Splitter->Unsplit();
 	    }
 
-	    int result = log->process(file);
-	    if (result < 0) {
-		// Error running cavern or processing data.
-		return;
-	    }
-
-	    AddToFileHistory(file);
-	    wxString file3d(file, 0, file.length() - 3);
-	    file3d.append(wxT("3d"));
-	    if (!LoadData(file3d, survey))
-		return;
-	    if (result == 0) {
-		InitialiseAfterLoad(file);
-	    }
+	    if (wxFileExists(file)) AddToFileHistory(file);
+	    log->process(file);
+	    // Log window will tell us to load file if it successfully completes.
 	    return;
 	}
     }
@@ -2012,6 +2006,7 @@ void MainFrm::OnShowLog(wxCommandEvent&)
     if (!m_Log) return;
     wxWindow * win = m_Splitter->GetWindow1();
     m_Splitter->ReplaceWindow(win, m_Log);
+    win->Show(false);
     if (m_Splitter->IsSplit()) {
 	m_SashPosition = m_Splitter->GetSashPosition(); // save width of panel
 	m_Splitter->Unsplit();
