@@ -174,7 +174,7 @@ class traverse : public vector<PointInfo> {
     double length;
     double E, H, V;
 
-    traverse() : n_legs(0), length(0), E(-1), H(-1), V(-1) { }
+    traverse() : n_legs(0), isSplay(false), length(0), E(-1), H(-1), V(-1) { }
 };
 
 class MainFrm : public wxFrame {
@@ -533,6 +533,7 @@ public:
     void DisplayTreeInfo(const wxTreeItemData* data = NULL);
     void TreeItemSelected(const wxTreeItemData* data, bool zoom);
     PresentationMark GetPresMark(int which);
+    bool Animating() const { return m_Gfx && m_Gfx->Animating(); }
 
 private:
     DECLARE_EVENT_TABLE()
