@@ -38,6 +38,7 @@
 #include "osdepend.h"
 #include "filelist.h"
 #include "debug.h"
+#include "str.h"
 
 #ifdef AVEN
 # include "aven.h"
@@ -542,7 +543,7 @@ add_unicode(int charset, unsigned char *p, int value)
 #endif
 #if OS_WIN32
    case CHARSET_DOSCP850: {
-      unsigned char uni2dostab[] = {
+      static const unsigned char uni2dostab[] = {
 	 255, 173, 189, 156, 207, 190, 221, 245,
 	 249, 184, 166, 174, 170, 240, 169, 238,
 	 248, 241, 253, 252, 239, 230, 244, 250,
@@ -562,7 +563,7 @@ add_unicode(int charset, unsigned char *p, int value)
       }
 #if 0
       if (value == 305) { /* LATIN SMALL LETTER DOTLESS I */
-	 *p = 213;
+	 *p = 213; /* "Modified CP850" has the Euro sign here. */
 	 return 1;
       }
       if (value == 402) { /* LATIN SMALL LETTER F WITH HOOK */
@@ -1150,6 +1151,20 @@ macosx_got_msg:
    select_charset(default_charset());
 }
 
+#if OS_WIN32 || OS_UNIX_MACOSX
+const char *
+msg_proj_finder(const char * file)
+{
+    static char * r = NULL;
+    static int r_len = 0;
+    s_zero(&r);
+    s_cat(&r, &r_len, pth_cfg_files);
+    s_cat(&r, &r_len, "/proj/");
+    s_cat(&r, &r_len, file);
+    return r;
+}
+#endif
+
 const char *
 msg(int en)
 {
