@@ -1,6 +1,6 @@
 /* netartic.c
  * Split up network at articulation points
- * Copyright (C) 1993-2003,2005,2012,2014 Olly Betts
+ * Copyright (C) 1993-2003,2005,2012,2014,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -389,8 +389,14 @@ articulate(void)
 	 /* Anonymous stations must be at the end of a trailing traverse (since
 	  * the same anonymous station can't be referred to more than once),
 	  * and trailing traverses have been removed at this point.
+	  *
+	  * However, we may removed a trailing traverse back to an anonymous
+	  * station.  FIXME: It's not helpful to fail to point to a station
+	  * in such a case - it would be much nicer to look through the list
+	  * of trailing traverses in such a case to find a relevant traverse
+	  * and then report a station name from there.
 	  */
-	 SVX_ASSERT(!TSTBIT(stn->name->sflags, SFLAGS_ANON));
+	 /* SVX_ASSERT(!TSTBIT(stn->name->sflags, SFLAGS_ANON)); */
 	 if (stn->name->ident) {
 	    if (!fNotAttached) {
 	       fNotAttached = fTrue;
@@ -405,19 +411,22 @@ articulate(void)
    }
 
    {
-      component *comp;
+      component *comp = component_list;
 
 #ifdef DEBUG_ARTIC
       printf("\nDump of %d components:\n", cComponents);
 #endif
-      for (comp = component_list; comp; comp = comp->next) {
+      while (comp) {
 	 node *list = NULL, *listend = NULL;
 	 articulation *art;
+	 component * old_comp;
 #ifdef DEBUG_ARTIC
 	 printf("Component:\n");
 #endif
 	 SVX_ASSERT(comp->artic);
-	 for (art = comp->artic; art; art = art->next) {
+	 art = comp->artic;
+	 while (art) {
+	    articulation * old_art;
 #ifdef DEBUG_ARTIC
 	    printf("  Articulation (%p):\n", art->stnlist);
 #endif
@@ -437,6 +446,9 @@ articulate(void)
 #endif
 	       listend = stn;
 	    }
+	    old_art = art;
+	    art = art->next;
+	    osfree(old_art);
 	 }
 #ifdef DEBUG_ARTIC
 	 putnl();
@@ -458,6 +470,10 @@ articulate(void)
 	 listend->next = stnlist;
 	 if (stnlist) stnlist->prev = listend;
 	 stnlist = list;
+
+	 old_comp = comp;
+	 comp = comp->next;
+	 osfree(old_comp);
       }
 #ifdef DEBUG_ARTIC
       printf("done articulating\n");
